#!/bin/bash

log_info() {
  echo "INFO: httpd: $@" >&2
}

CONFIG_FILE="/etc/httpd/conf/httpd.conf"
CONFIG_PATH="/etc/httpd"
DATA_DIR="./httpd-data"
DIRTYCONF_D="/root/preupgrade/dirtyconf"

# copy httpd configuration files stored at dirtyconf directory to expected
# location on new system
log_info "Restore the original configuration from backed up data."
cp -ar "$DIRTYCONF_D/$CONFIG_PATH" "$(dirname "$CONFIG_PATH")"
restorecon -r "$CONFIG_PATH"

if [ ! -f "$CONFIG_FILE" ]; then
    echo "Error: The $CONFIG_FILE file does not exist." >&2
    exit 1
fi

# now fix the config files, so the server will be able to run
log_info "Modify the configuration to be compatible with the new httpd."
grep -qi "Include conf.modules.d/\\*.conf" $CONFIG_FILE
if [ $? -ne 0 ]; then
    sed -i -e '/^ServerRoot/{s/$/\nInclude conf.modules.d\/\*.conf/}' $CONFIG_FILE
fi

line=
while IFS= read line || [ -n "$line" ];
do
    grep -qi "^[ \t]*LoadModule.*$line" $CONFIG_FILE
    if [ $? -eq 0 ]; then
        sed -i -e "/^[ \t]*LoadModule.*$line/d" $CONFIG_FILE
    fi
done <"$DATA_DIR/default_modules"

line=
while IFS= read line || [ -n "$line" ];
do
    grep -qi "^[ \t]*LoadModule.*$line" $CONFIG_FILE
    if [ $? -eq 0 ]; then
        sed -i -e "/^[ \t]*LoadModule.*$line/d" $CONFIG_FILE
    fi
done <"$DATA_DIR/removed_modules"

grep -qi "^[ \t]*#LoadModule speling_module" $CONFIG_FILE && \
grep -qi "^[ \t]*CheckSpelling" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    sed -i -e 's/#LoadModule speling_module/LoadModule speling_module/' $CONFIG_FILE
fi

grep -qi "^[ \t]*#LoadModule usertrack_module" $CONFIG_FILE && \
grep -qi "^[ \t]*Cookie.*" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    sed -i -e 's/#LoadModule usertrack_module/LoadModule usertrack_module/' $CONFIG_FILE
fi

grep -qi "^[ \t]*HTTPD=.*worker.*" /etc/sysconfig/httpd
if [ $? -eq 0 ]; then
    sed -i -e 's/#LoadModule mpm_worker_module/LoadModule mpm_worker_module/' $CONFIG_PATH/conf.modules.d/00-mpm.conf
    sed -i -e 's/LoadModule mpm_prefork_module/#LoadModule mpm_prefork_module/' $CONFIG_PATH/conf.modules.d/00-mpm.conf
fi

grep -qi "^[ \t]*HTTPD=.*event.*" /etc/sysconfig/httpd
if [ $? -eq 0 ]; then
    sed -i -e 's/#LoadModule mpm_event_module/LoadModule mpm_event_module/' $CONFIG_PATH/conf.modules.d/00-mpm.conf
    sed -i -e 's/LoadModule mpm_prefork_module/#LoadModule mpm_prefork_module/' $CONFIG_PATH/conf.modules.d/00-mpm.conf
fi

grep -qi "^[ \t]*LoadModule perl_module" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    grep -qi "^[ \t]*Perl.*" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
    if [ $? -ne 0 ]; then
        ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's/LoadModule perl_module/#LoadModule perl_module/'
    fi
fi

grep -qi "^[ \t]*LoadModule dnssd_module" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    grep -qi "^[ \t]*DNSSDEnable" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
    if [ $? -ne 0 ]; then
        ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's/LoadModule dnssd_module/#LoadModule dnssd_module/'
    fi
fi

grep -qi "^[ \t]*LoadModule auth_pgsql_module" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    grep -qi "^[ \t]*Auth_PG.*" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
    if [ $? -ne 0 ]; then
        ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's/LoadModule auth_pgsql_module/#LoadModule auth_pgsql_module/'
    fi
fi

grep -qi "^[ \t]*LoadModule mysql_auth_module" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    grep -qi "^[ \t]*AuthMySQL.*" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
    if [ $? -ne 0 ]; then
        ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's/LoadModule mysql_auth_module/#LoadModule mysql_auth_module/'
    fi
fi

grep -qi "^[ \t]*SSLMutex default" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's/SSLMutex default/#SSLMutex default/'
fi

grep -qi "^[ \t]*SSLPassPhraseDialog[ \t]*builtin" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's|SSLPassPhraseDialog[ \t]*builtin|SSLPassPhraseDialog exec:/usr/libexec/httpd-ssl-pass-dialog|'
fi

grep -qi "^[ \t]*SSLSessionCache[ \t]*shmcb:/var/cache/mod_ssl/scache" $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf
if [ $? -eq 0 ]; then
    ls $CONFIG_FILE $CONFIG_PATH/conf.d/*.conf | xargs sed -i -e 's|shmcb:/var/cache/mod_ssl/scache|shmcb:/run/httpd/sslcache|'
fi

log_info "The modification of configuration files has been finished."

